#include <mpi.h>
#include "tests.hpp"
#include "libcDAG.hpp"

void init_buffer(void* buffer, size_t buffer_size) {
	
	srand(time(NULL));
	
	for (size_t i=0; i<buffer_size; i++) {
		char rnd = rand() % 256;
		((char*) buffer)[i] = rnd;
	}

}

int main(int argc, char** argv) {

	MPI_Init(&argc, &argv);
	TEST_INIT("linear broadcast");

	cDAG_Init();

	const int datasize = 8;
	char* buffer_cdag = (char*) malloc(datasize);
	char* buffer_mpi = (char*) malloc(datasize);
	init_buffer(buffer_cdag, datasize);
	init_buffer(buffer_mpi, datasize);

	int r = cDAG_MyRank();
	int p = cDAG_GroupSize();

for (int i=0; i<1000; i++) {
	cDAG_Graph g = cDAG_Create_graph();

	if (r == 0) {
		for (int i=1; i<p; i++) {
			cDAG_Send(g, buffer_cdag, datasize, i);
		}
	}
	else {
		cDAG_Recv(g, buffer_cdag, datasize, 0);
	}

	g->do_coll_detection = 0;
	cDAG_Schedule sched = cDAG_Compile(g);
	cDAG_FreeGraph(g);
	cDAG_Handle hndl = cDAG_Run(sched);
	cDAG_Wait(hndl);
	cDAG_FreeSchedule(sched);

}

	MPI_Bcast(buffer_mpi, datasize, MPI_BYTE, 0, MPI_COMM_WORLD);

	if (memcmp(buffer_mpi, buffer_cdag, datasize) != 0) TEST_FAILED();
	else TEST_SUCCESSFULL();

	cDAG_Finalize();
	MPI_Finalize();

}
