#include <mpi.h>
#include "tests.hpp"
#include "libcDAG.hpp"

struct myop_args {
	int* inp1;
	int* inp2;
	int* outp;
};

void myop_func(void* args, void* scratchpad_mem) {
	struct myop_args* ma = (struct myop_args*) args;
	*(ma->outp) = *(ma->inp1) > *(ma->inp2) ? *(ma->inp1) : *(ma->inp2);
}

int main(int argc, char** argv) {

	MPI_Init(&argc, &argv);
	TEST_INIT("user defined operation");

	cDAG_Init();

	int r = cDAG_MyRank();
	int p = cDAG_GroupSize();
	int a=r;
	int b=0;

	cDAG_Graph g = cDAG_Create_graph();
	cDAG_Opcode myop = cDAG_RegisterOp(g, &myop_func, sizeof(struct myop_args));

	if (r == 0) {
		for (int i=1; i<p; i++) {
			cDAG_Vertex recv = cDAG_Recv(g, &a, sizeof(int), i);
			struct myop_args margs;
			margs.inp1 = &a;
			margs.inp2 = &b;
			margs.outp = &b;
			cDAG_Vertex op = cDAG_Op(g, myop, &margs);

			// predefined ops would work like
			// const struct bly={ .invec=bla, .outvec=blub, .size=2 };
			// cDAG_Op(cDAG_INT_MAX, &bly);

			cDAG_Requires(g, recv, op);
		}
	}
	else {
		cDAG_Vertex send = cDAG_Send(g, &a, sizeof(int), 0);
	}

	// disable coll detection for now
	g->do_coll_detection = 0;

	cDAG_Schedule sched = cDAG_Compile(g);
	cDAG_FreeGraph(g);
	cDAG_Handle hndl = cDAG_Run(sched);
	cDAG_Wait(hndl);

	cDAG_FreeSchedule(sched);

	if ((r == 0) && (b != p-1)) TEST_FAILED();
	else TEST_SUCCESSFULL();

	cDAG_Finalize();
	MPI_Finalize();

}

